/********************************************************************
 FileName:    	GPIO.c
 Dependencies: 	aguijon.h
 Hardware:	Aguijon rev3.0 & rev4.0
 Complier:  	Microchip XC16, C30
 Company:	Vinagrn Digital
 Author:	Juan Carlos Orozco Gutierrez

 Software License Agreement:

 This software has been licensed under the GNU General Public
 License is intended to guarantee your freedom to share and change free
 software--to make sure the software is free for all its users.

********************************************************************
 File Description:
    Low level General Purpose Input / Output + i2c & SPI Support File


 Change History:
  Rev   Description                                 Modified by:
  ----  -----------------------------------------  --------------
  1.0   Initial release                             JCOG
  2.0   Updated with [VD] HammerHead libraries      JCOG
  2.1   Rewrote all SPI functions                   JCOG

 For extra documentation and support:
 *  http://www.vinagrondigital.com
********************************************************************/
#include "aguijon.h"
#include "IO.h"
#include <i2c.h>
#include <PPS.h>
#include <outcompare.h>

/*
 * Config Fuses -DO NOT REMOVE-
 * otherwise, HammerHead and pretty much everything else won't work
 */
_CONFIG1(JTAGEN_OFF & FWDTEN_OFF & ICS_PGx2)
_CONFIG2(FNOSC_PRIPLL & POSCMOD_HS& PLL_96MHZ_ON & PLLDIV_DIV2 & FCKSM_CSECMD & IOL1WAY_OFF)
///////////////////////////////////////////////////////////////////////////////

/*GPIO*/
///////////////////////////////////////////////////////////////////////////////
void IO_Init(void)
{
#if defined(AGUIJON_REV_3)
    TRISB = 0xBDCF; //configure PORTB
    LATB  = 0x0000;

    TRISD = 0x0000; //configure PORTD
    LATD  = 0x0000;

    TRISE = 0x0000; //configure PORTE
    LATE  = 0x0000;

    TRISF = 0x0000; //configure PORTF
    LATF  = 0x0000;
    
    TRISG = 0x0000; //configure PORTG
    LATG  = 0x0000;

#elif defined(AGUIJON_REV_4) || defined(AGUIJON_REV_41)
    TRISA = 0x0200; //configure PORTA
    LATA  = 0x0000;

    TRISB = 0xFF7F; //configure PORTB
    LATB  = 0x0000;

    TRISC = 0x0008; //configure PORTC
    LATC  = 0x0000;

    TRISD = 0x0000; //configure PORTD
    LATD  = 0x0000;

    TRISE = 0x0300; //configure PORTE
    LATE  = 0x0000;

    TRISF = 0x0000; //configure PORTF
    LATF  = 0x0000;

    TRISG = 0x0000; //configure PORTG
    LATG  = 0x0000;

#endif

    WDOG_SoftDisable();         //Disable external watchdog just in case it is set (HW)

    USB_5V_FAULT_TRIS  = 1;     //USB fault pin as an input
    USB_5V_ENABLE      = 1;     //disable USB 5V output

    /* Select analog input, just in case user doesnt call ADC_Init(); */
    AD1PCFG = ADC_PIN_MASK;

    /* Disable DIP switch, just in case user doenst call DIPSW_Config();*/
    DIPSW_ENABLE_LSB_PIN = 1;
    DIPSW_ENABLE_MSB_PIN = 1;
}

void IO_PWMinit(void)
{
    /*initializes Peripheral Pin Select Features such as:
     * 
     * LCD Backlight PWM,
     * Buzzer PWM,
     *
     */

    /* RP21 -> PWM Output Compare 1 (LCD backlight) */
    /* RP26 -> PWM Output Compare 2 (Buzzer) */
    PPSUnLock;  //unlock PPS register

    RPOR10bits.RP21R = 18;  //assign PWM1 (OC1) to RP21
    RPOR13bits.RP26R = 19;  //assign PWM2 (OC2) to RP26
   
    PPSLock;    //lock PPS register

    CloseOC1();                 //disable OC1
    CloseOC2();                 //disable OC2
    ConfigIntOC1(OC_INT_OFF);   //disable interrupt
    ConfigIntOC2(OC_INT_OFF);   //disable interrupt
}

/*SPI, Implemented by default @expPort #2*/
///////////////////////////////////////////////////////////////////////////////
void IO_SPIinit(void)
{
    SPI1_MOSI_TRIS  = 0;
    SPI1_MISO_TRIS  = 1;
    SPI1_SCK_TRIS   = 0;
    SPI1_CS_TRIS    = 0;

    PPSUnLock;  //unlock PPS register

    RPINR20bits.SDI1R   = SPI1_MISO_REMAP;//assign MISO to MISO pin
    SPI1_MOSI_REMAP     = 7;    //remap MOSI
    SPI1_SCK_REMAP      = 8;    //remap SCK
    //refer to "PPS.h" for Function values.

    PPSLock;    //lock PPS register

    //SPI register config
    SPI1CON1 = 0x013B;
    SPI1CON2 = 0;
    SPI1STAT = 0x8000;  //enable SPI module
    INTCON2bits.INT1EP = 1;
}

void IO_SPIsendByte(BYTE data)
{
    BYTE i;
    int retries = 5000;

    IFS0bits.SPI1IF = 0;
    i = SPI1BUF;
    SPI1BUF = data;
    while((!IFS0bits.SPI1IF) && retries){ retries--; }
}

BYTE IO_SPIreadByte(void)
{
    IO_SPIsendByte(0x00);
    return SPI1BUF;
}

void IO_SPIChipSelectSet(BOOL state)
{
    if(state == ON){
        SPI1_CS = 0;    //select device
    }else{
        SPI1_CS = 1;    //deselect device
    }
}
///////////////////////////////////////////////////////////////////////////////

/*I2C*/
///////////////////////////////////////////////////////////////////////////////
void IO_I2Cinit(void)
{
    unsigned int config2, config1;

    /* Baud rate is set for 100 Khz */
    config2 = 0x9D;

    /* Configure I2C for 7 bit addressing mode */
    config1 = (I2C_ON & I2C_IDLE_CON & I2C_CLK_HLD & I2C_IPMI_DIS & I2C_7BIT_ADD \
             & I2C_SLW_DIS & I2C_SM_DIS & I2C_GCALL_DIS & I2C_STR_DIS & I2C_NACK \
             & I2C_ACK_DIS & I2C_RCV_DIS & I2C_STOP_DIS & I2C_RESTART_DIS & I2C_START_DIS);

#if defined(AGUIJON_REV_3)
    I2C1CON = config1;
    I2C1BRG = config2;

    IdleI2C1(); //wait until I2C bus is inactive
#elif defined(AGUIJON_REV_4) || defined(AGUIJON_REV_41)
    I2C2CON = config1;
    I2C2BRG = config2;

    IdleI2C2(); //wait until I2C bus is inactive
#endif
}

void IO_I2Cstart(void)
{
    #if defined(AGUIJON_REV_3)
        I2C1CONbits.SEN = 1;    // Initiate Start on SDA and SCL pins
        while(I2C1CONbits.SEN); // Wait till start sequence is completed
    #elif defined(AGUIJON_REV_4) || defined(AGUIJON_REV_41)
        I2C2CONbits.SEN = 1;    // Initiate Start on SDA and SCL pins
        while(I2C2CONbits.SEN); // Wait till start sequence is completed
    #endif
}

void IO_I2Cstop(void)
{
    #if defined(AGUIJON_REV_3)
        I2C1CONbits.PEN = 1;    // Initiate Stop on SDA and SCL pins
        while(I2C1CONbits.PEN); // Wait till stop sequence is completed
    #elif defined(AGUIJON_REV_4) || defined(AGUIJON_REV_41)
        I2C2CONbits.PEN = 1;    // Initiate Stop on SDA and SCL pins
        while(I2C2CONbits.PEN); // Wait till stop sequence is completed
    #endif
}

void IO_I2CsendByte(BYTE data)
{
    #if defined(AGUIJON_REV_3)
        MasterWriteI2C1(data);      // Write Slave address and set master for transmission
        /* Wait till address is transmitted */
        while(I2C1STATbits.TBF);    // 8 clock cycles
        while(!IFS1bits.MI2C1IF);   // Wait for 9th clock cycle
        IFS1bits.MI2C1IF = 0;       // Clear interrupt flag
    #elif defined(AGUIJON_REV_4) || defined(AGUIJON_REV_41)
        MasterWriteI2C2(data);      // Write Slave address and set master for transmission
        /* Wait till address is transmitted */
        while(I2C2STATbits.TBF);    // 8 clock cycles
        while(!IFS3bits.MI2C2IF);   // Wait for 9th clock cycle
        IFS3bits.MI2C2IF = 0;       // Clear interrupt flag
    #endif
    delayus(10);                // when MCU is at 16MHz, this delay is needed to support LCD
}

BYTE IO_I2CreadByte(BOOL ack)
{
    BYTE data=0;

    #if defined(AGUIJON_REV_3)
        I2C1CONbits.RCEN   = 1;
        while(I2C1CONbits.RCEN);
        I2C1CONbits.RCEN   = 0;
        I2C1STATbits.I2COV = 0;
        data = I2C1RCV;

        if(ack){
            /*Ack I2C*/
            I2C1CONbits.ACKDT = 0;
            I2C1CONbits.ACKEN = 1;
            while(I2C1CONbits.ACKEN);
        }
    #elif defined(AGUIJON_REV_4) || defined(AGUIJON_REV_41)
        I2C2CONbits.RCEN   = 1;
        while(I2C2CONbits.RCEN);
        I2C2CONbits.RCEN   = 0;
        I2C2STATbits.I2COV = 0;
        data = I2C2RCV;

        if(ack){
            /*Ack I2C*/
            I2C2CONbits.ACKDT = 0;
            I2C2CONbits.ACKEN = 1;
            while(I2C2CONbits.ACKEN);
        }
    #endif
    return data;
}

void IO_I2CsendNACK(void)
{
    #if defined(AGUIJON_REV_3)
        /*Not Ack I2C*/
        I2C1CONbits.ACKDT = 1;
        I2C1CONbits.ACKEN = 1;
    #elif defined(AGUIJON_REV_4) || defined(AGUIJON_REV_41)
        /*Not Ack I2C*/
        I2C2CONbits.ACKDT = 1;
        I2C2CONbits.ACKEN = 1;
    #endif
}
///////////////////////////////////////////////////////////////////////////////
