/********************************************************************
 FileName:    	main.c
 Dependencies: 	aguijon.h
 Hardware:	Aguijon rev3.0, rev4.0 & rev4.1
 Complier:  	Microchip XC16, C30
 Company:	Vinagrón Digital
 Author:	Juan Carlos Orozco Gutierrez

 Software License Agreement:

 This software has been licensed under the GNU General Public
 License is intended to guarantee your freedom to share and change free
 software--to make sure the software is free for all its users.

********************************************************************
 File Description:
    Main Application Note 28 file

 Change History:
  Rev   Description                                 Modified by:
  ----  -----------------------------------------  --------------
  1.0   Initial release                             DIRD

 For extra documentation and support:
 *  http://www.vinagrondigital.com
********************************************************************/
#include "aguijon.h"
#include "analog.h"             //analog support
#include "EEPROM.h"             //EEPROM support
#include "IO.h"                 //IO support
#include "inputs.h"             //inputs support
#include "LCD.h"                //LCD support
#include "outputs.h"            //output support
#include "realtime.h"           //RTCC module support
#include "RS232.h"              //RS232 support
#include "timers.h"             //timers support

//USB Support//
#include "USB/usb.h"
#include "USB/usb_host_msd.h"
#include "USB/usb_host_msd_scsi.h"
#include "MDD File System/FSIO.h"

#define CHUNK_SIZE 15
///////////////////////////////////////

void HammerHead_Init(void)
{
    /*GPIO*/
    IO_Init();        //initialize all ports
    IO_PWMinit();     //initialize HammerHead PWMs

    /*Inputs*/
    DIPSW_Config(DIPSW_DISABLE_CMD);
    ADC_Init();

    /*UART*/
    RS232_Init(BAUD19200);  //set 19200bps as UART speed

    /*LCD*/
    LCD_Init(LCD_MODE_1);

#if defined(USE_LCD_EXTRA_FEATURES)
    LCD_BacklightFadeIn();
#else
    LCD_BacklightSet(BLIGHT_LVL_4);
#endif

    /*RTCC*/
    RTCC_Init();    //interurpt is handled within realtime.c
    RTCC_Enable();

    /*timers*/
    Timer1_Init();
    Timer1_SetTick(TIMER_TICK_50MS);
    Timer1_Enable();        //interurpt is handled within timers.c

    Timer4_Init();
    Timer4_SetTick(TIMER_TICK_100MS);
    Timer4_Enable();        //interurpt is handled within timers.c

    LEDport_Set(0,ON);
}

void InsertChime(void)
{
    delayms(100);
    Buzzer_On(800,50);
    delayms(30);
    Buzzer_Off();
    delayms(80);
    Buzzer_On(421,50);
    delayms(30);
    Buzzer_Off();
}

void RemoveChime(void)
{
    delayms(100);
    Buzzer_On(421,50);
    delayms(30);
    Buzzer_Off();
    delayms(80);
    Buzzer_On(800,50);
    delayms(30);
    Buzzer_Off();
}

static int File_GetSize(FSFILE *inFile)
{
    int fileSize = 0;

    FSfseek(inFile,0,SEEK_END);
    fileSize = FSftell(inFile);
    FSfseek(inFile,0,SEEK_SET);

    return fileSize;
}

char TxBufferFull (void)
{
    return(U2STAbits.UTXBF);
}

///////////////////////////////////////


int main(void)
{

    char rs232MSG[CHUNK_SIZE];
    unsigned int units_read , i , unit , file_size;

    FSFILE * myFile;

    HammerHead_Init();  //initialize [VD]HammerHead

    USBInitialize(0);   //Initialize the stack
    USB_5V_ENABLE = 0;  //Turn 5V USB output on

    LCD_IntroAnimation();
    LCD_PutStr(1,0,"Vinagron Digital",TRUE);
    LCD_PutStr(2,0,"Application Note 28",FALSE);
    delayms(1000);
    LCD_PutStr(1,0,"Aguijon USB MSD",TRUE);
    LCD_PutStr(2,0,"Insert flash drive",FALSE);

    for(;;){
        //USB stack process function
        USBTasks();

        //if thumbdrive is plugged in
        if(USBHostMSDSCSIMediaDetect()){
            InsertChime();
            //now a device is attached

            //See if the device is attached and in the right format
            if(FSInit()){
                //Opening a file in mode "r"
                //The device must have the file with the same name
                myFile = FSfopen("msg.txt","r");

                //Get file size
                file_size=File_GetSize(myFile);

                for(i=0;i<file_size;i+=CHUNK_SIZE)
                {
                    //Change the current position in a file
                    FSfseek(myFile,i,SEEK_SET);

                    //Read some data from the file
                    units_read = FSfread(rs232MSG,1,CHUNK_SIZE,myFile);

                    for(unit=0;unit<units_read;unit++)
                    {
                        //wait if the buffer is full
                        while(TxBufferFull());
                        //Create a strings for the RS232//
                        RS232_Send(rs232MSG[unit]);
                    }
                }

                //Always make sure to close the file.
                FSfclose(myFile);

                //Just sit here until the device is removed.
                LCD_PutStr(1,0,"DONE!",TRUE);
                LCD_PutStr(2,0,"Safe to remove USB",FALSE);
                while(USBHostMSDSCSIMediaDetect()){
                    USBTasks();
                }
                RemoveChime();
                //Create a strings for the LCD//
                LCD_PutStr(1,0,"Aguijon USB MSD",TRUE);
                LCD_PutStr(2,0,"Insert flash drive",FALSE);
            }
        }
    }
    return 0;
}